//+------------------------------------------------------------------+
//|                                                   TradingLib.mq4 |
//|                                 Copyright  2006, Trader Phoenix |
//|                                             http://www.udaff.com |
//+------------------------------------------------------------------+
#property copyright "Copyright  2006, Trader Phoenix"
#property link      "http://www.udaff.com"

#include <PxLib.mqh>
#include <MTSReport.mqh>
#define CMD_ORDERCANCEL  -99999

//+------------------------------------------------------------------+
//| defines                                                          |
//+------------------------------------------------------------------+
// #define MacrosHello   "Hello, world!"
// #define MacrosYear    2005

//+------------------------------------------------------------------+
//| DLL imports                                                      |
//+------------------------------------------------------------------+
// #import "user32.dll"
//   int      SendMessageA(int hWnd,int Msg,int wParam,int lParam);

// #import "my_expert.dll"
//   int      ExpertRecalculate(int wParam,int lParam);
// #import

//+------------------------------------------------------------------+
//| EX4 imports                                                      |
//+------------------------------------------------------------------+
// #import "stdlib.ex4"
//   string ErrorDescription(int error_code);
// #import
//+------------------------------------------------------------------+


int MagicMA( int _ma )
{
   return( _ma+Period() );
}

string BuyComent( string _mtsName )
{
   return( "By(#"+_mtsName+")" );
}

string SellComent( string _mtsName )
{
   return( "Sell(#"+_mtsName+")" );
}


//+------------------------------------------------------------------+
//| Calculate optimal lot size                                       |
//+------------------------------------------------------------------+

// 0 - No optivization
// 1 - Minus
// 2 - Mul
// 3 - STD Percent
// 4 - STD Percent from MAX

string PXS_GetGlobalVarLO()
{
   if( IsTesting() )
   {
      return( "Test_"+"AbsoluteBalance"+AccountNumber() );
   }
   else
   {
      return( "AbsoluteBalance"+AccountNumber() );
   }
}

double LotsOptimized( int _magicMa,int OptimizeLotMethod, double MaximumRisk, int DecreaseFactor )
{
   double lot=0;
   double lot1=0;
   int i;
   int    orders=HistoryTotal();     // history orders total
   int    losses=0;                  // number of losses orders without a break
   
   if( OptimizeLotMethod == 0 )
   {
      lot = MaximumRisk;
      if(lot<0.1) lot=0.1;
      return( NormalizeDouble(lot,1) );
   }
   
//---- select lot size
   if( OptimizeLotMethod==4 )
   {
      // Get acc Balance
      double _accBalance = GlobalVariableGet( PXS_GetGlobalVarLO() );
      if( AccountBalance()>_accBalance )
      {
         _accBalance = AccountBalance();
         GlobalVariableSet( PXS_GetGlobalVarLO(),_accBalance );
      }
      
      // Calc lot
      lot = NormalizeDouble(_accBalance*MaximumRisk/1000.0,1);
   }
   else
   {
      lot=NormalizeDouble(AccountBalance()*MaximumRisk/1000.0,1);
   }
   lot1=lot;
   
//---- calcuulate number of losses orders without a break
   if(DecreaseFactor>0)
   {
      // Lot on whole portfolio
      for(i=orders-1;i>=0;i--)
      {
         if(OrderSelect(i,SELECT_BY_POS,MODE_HISTORY)==false) { Print("Error in history!"); break; }
         if(OrderType()>OP_SELL) continue;
         //----
         if(OrderProfit()>0) break;
         if(OrderProfit()<0) losses++;
      }
      
      switch( OptimizeLotMethod )
      {
         case 1:
            if(losses>1) lot=NormalizeDouble(lot-lot*losses/DecreaseFactor,1);
            break;
         case 2:
            if(losses>1) lot=NormalizeDouble(lot/(losses*DecreaseFactor),1);
            break;
         case 3:
         case 4:
            lot=NormalizeDouble(lot,1);
            break;
      }
      
      // Lot on the system only
      losses=0;
      for(i=orders-1;i>=0;i--)
      {
         if(OrderSelect(i,SELECT_BY_POS,MODE_HISTORY)==false) { Print("Error in history!"); break; }
         if(OrderSymbol()!=Symbol() || OrderType()>OP_SELL || OrderMagicNumber()!=MagicMA(_magicMa) ) continue;
         //----
         if(OrderProfit()>0) break;
         if(OrderProfit()<0) losses++;
      }
      
      switch( OptimizeLotMethod )
      {
         case 1:
            if(losses>1) lot1=NormalizeDouble(lot-lot*losses/DecreaseFactor,1);
            break;
         case 2:
            if(losses>1) lot1=NormalizeDouble(lot/(losses*DecreaseFactor),1);
            break;
         case 3:
         case 4:
            lot1=NormalizeDouble(lot1,1);
            break;
      }
   }
//---- return lot size
   lot = MathMin(lot,lot1);
   if(lot<0.1) lot=0.1;
   return(lot);
}

bool CheckOpened( int _magicMa )
{
   for(int i=0;i<OrdersTotal();i++)
   {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false) break;
      if( OrderMagicNumber()!=MagicMA(_magicMa) || OrderSymbol()!=Symbol()) continue;
      switch( OrderType() )
      {
         case OP_BUY:
         case OP_SELL: return( TRUE );
      }
   }
   
   return( false );
}

void AllowTrade()
{
   while( IsTradeContextBusy() )
      Sleep( 1000 );
}


void CancelAllOrders( int _ma )
{
   // Cansel All in Current Symbol
   int ticket=0;
   bool bDeleted = FALSE;
   bool bFound;
   int  count=OrdersTotal();
   int _magicMa = MagicMA(_ma);
   
   // While orders are deleted
   while( ticket<count )
   {
      // Init deteion
      bDeleted = FALSE;
      count=OrdersTotal();
      ticket = 0;
      
      // Delete one order
      while(ticket<count && !bDeleted)
      {
         if( OrderSelect(ticket,SELECT_BY_POS) )
         {
            if( OrderSymbol()==Symbol() && (OrderType() == OP_BUYLIMIT || OrderType() == OP_BUYSTOP || OrderType() == OP_SELLLIMIT || OrderType() == OP_SELLSTOP ) )
            {
               if( _ma == 0 || OrderMagicNumber() == _magicMa )
               {
                  if( OrderDelete(OrderTicket()) )
                  {
                     bDeleted = TRUE;
                  }
                  else
                  {
                     int err = GetLastError();
                     Alert("Can not delete order ("+OrderTicket()+"): ",ErrorDescription(err) );
                  }
               }
            }
         }
         ticket++;
      }
      
      // If nothing deleted - exit while
      if( !bDeleted )
      {
         ticket=99999;
      }
   }
}

//+------------------------------------------------------------------+
//| Traling Stop                                                     |
//+------------------------------------------------------------------+

// Model:
// 0 - No
// 1 - Frahtal
// 2 - Bars
// 3 - MA
// 4 - Fixed from close
// 5 - ATR

void TrailingMove( string _mtsName, int _ma, int TrailingModel, int _period, int _tralDelta, int _paritet, double _k=0 )
{
   if(Volume[0]>1) return;
   
   int    res;
   int    i_Frahtal;
   int    pos=1;
   double dStop = 0;
   
   for(int i=0;i<OrdersTotal();i++)
   {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false)        break;
      if( OrderMagicNumber()!=MagicMA(_ma) || OrderSymbol()!=Symbol()) continue;
      
      switch( OrderType() )
      {
         case OP_BUY:
            switch( TrailingModel )
            {
               case 0: // No
                  break;
               case 1: // Frahtal
                  i_Frahtal = PX_FindFrahtalDown();
                  if( i_Frahtal != 0 )
                  {
                     dStop = Low[i_Frahtal]-_tralDelta*Point;
                  }
                  break;
               case 2: // Bars
                  dStop = iLow( Symbol(),Period(),Lowest(Symbol(),Period(),MODE_LOW,_period+1,0 ) ) - _tralDelta*Point;
                  break;
               case 3: // Ma
                  dStop = iMA( NULL,0,_period,0,MODE_EMA,PRICE_MEDIAN,0 )-(_tralDelta-PX_GetSpread(Symbol()))*Point;
                  break;
               case 4: // Fixed from close
                  dStop = Close[1] - _tralDelta*Point;
                  break;
               case 5: // ATR
                  dStop = Close[1] - iATR( NULL,0,_period,1 )*_k;
                  break;
            }
            
            // Check if effective
            if( OrderStopLoss() >= dStop ) dStop = 0;
            
            // Check Paritet
            if( OrderOpenPrice()+_paritet*Point < Close[0] )
            {
               if( OrderOpenPrice() >= dStop && OrderOpenPrice()>=OrderStopLoss() )
                  dStop = OrderOpenPrice();
            }
            break;
         case OP_SELL:
            switch( TrailingModel )
            {
               case 0: // No
                  break;
               case 1: // Frahtal
                  i_Frahtal = PX_FindFrahtalUP();
                  if( i_Frahtal != 0 )
                  {
                     dStop = High[i_Frahtal]+(_tralDelta+PX_GetSpread(Symbol()))*Point;
                  }
                  break;
               case 2: // Bars
                  dStop = iHigh( Symbol(),Period(),Highest(Symbol(),Period(),MODE_HIGH,_period+1,0 )) + (_tralDelta+PX_GetSpread(Symbol()))*Point;
                  break;
               case 3: // Ma
                  dStop = iMA( NULL,0,_period,0,MODE_EMA,PRICE_MEDIAN,0 )+_tralDelta*Point;
                  break;
               case 4: // Fixed from close
                  dStop = Close[1] + (_tralDelta+PX_GetSpread(Symbol()))*Point;
                  break;
               case 5: // ATR
                  dStop = Close[1] + iATR( NULL,0,_period,1 )*_k+PX_GetSpread(Symbol())*Point;
                  break;
            }
            
            // Check if effective
            if( OrderStopLoss()!=0 && dStop>=OrderStopLoss() ) dStop = 0;
            
            // Check Paritet
            if( OrderOpenPrice()-_paritet*Point-PX_GetSpread(Symbol())*Point > Close[0] )
            {
               if( OrderOpenPrice() <= dStop && (OrderOpenPrice()<=OrderStopLoss()||OrderStopLoss()==0) )
                  dStop = OrderOpenPrice();
            }
            
            break;
      }

      // Set Stop
      if( dStop != 0 )
      {
         AllowTrade();
         res = OrderModify(OrderTicket(),OrderOpenPrice(),dStop,OrderTakeProfit(),OrderExpiration(),CLR_NONE);
         ReportAction(OrderTicket() + " stop moved " + dStop);
         ReportOrder(res);
      }
   }
}

//+------------------------------------------------------------------+
//| Profit                                                           |
//+------------------------------------------------------------------+

// Model
// 0 - No
// 1 - SlverChannel
// 2 - Bollinger
// 3 - MA
// 4 - Keltner

void ProfitMove( string _mtsName, int _ma, int ProfitModel, int _period, int _profit, double _level )
{
   if(Volume[0]>1) return;
   
   int    res;
   double dProfit = 0;
   double dProfit1 = 0;

   double d_PU;
   double d_PL;
   
   // Special preparations
   switch( ProfitModel )
   {
      case 1:
         double d_sch_Upper = iCustom( NULL,0,"Silver-channels",_period,4,1);
         double d_sch_Lower = iCustom( NULL,0,"Silver-channels",_period,5,1);
         double d_sch_Mid = (d_sch_Upper + d_sch_Lower)/2;
      
         d_PU = d_sch_Mid + (d_sch_Upper - d_sch_Mid)*_level;
         d_PL = d_sch_Mid + (d_sch_Lower - d_sch_Mid)*_level;
         break;
      case 2:
         double d_BolU    = iBands( NULL,0, _period,2,0,PRICE_MEDIAN,MODE_UPPER,1 );
         double d_BolL    = iBands( NULL,0, _period,2,0,PRICE_MEDIAN,MODE_LOWER,1 );
         double d_BolM    = (d_BolU + d_BolL)/2;
         d_PU = d_BolM + (d_BolU - d_BolM)*_level;
         d_PL = d_BolM - (d_BolM - d_BolL)*_level;
         break;
   }
   
   for(int i=0;i<OrdersTotal();i++)
   {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false)        break;
      if( OrderMagicNumber()!=MagicMA(_ma) || OrderSymbol()!=Symbol()) continue;
      
      switch( OrderType() )
      {
         case OP_BUY:
            switch( ProfitModel )
            {
               case 0:
                  break;
               case 1:
               case 2:
                     dProfit = d_PU;
                  break;
               case 3: // Ma
                     dProfit1 = iMA( NULL,0,_period,0,MODE_EMA,PRICE_MEDIAN,0 )+_profit*Point;
                     if( dProfit1>OrderTakeProfit() )
                        dProfit=dProfit1;
                  break;
               case 4: // Keltner
                     dProfit = iCustom( NULL,0,"KeltnerChannel",_period,_profit,_level,0,1 );
                  break;
            }
            break;
         case OP_SELL:
            switch( ProfitModel )
            {
               case 0:
                  break;
               case 1:
               case 2:
                     dProfit = d_PL;
                  break;
               case 3: // Ma
                     dProfit1 = iMA( NULL,0,_period,0,MODE_EMA,PRICE_MEDIAN,0 )-(_profit-PX_GetSpread(Symbol()))*Point;
                     if( dProfit1<OrderTakeProfit() )
                        dProfit=dProfit1;
                  break;
               case 4: // Keltner
                     dProfit = iCustom( NULL,0,"KeltnerChannel",_period,_profit,_level,1,1 )+PX_GetSpread(Symbol())*Point;
                  break;
            }
            break;
      }

      // Set Stop
      if( dProfit == CMD_ORDERCANCEL )
      {
         AllowTrade();
         OrderClose(OrderTicket(),OrderLots(),OrderOpenPrice(),3);
         ReportAction(OrderTicket() + " order closed ");
      }
      else if( dProfit != 0 )
      {
         AllowTrade();
         res = OrderModify(OrderTicket(),OrderOpenPrice(),OrderStopLoss(),dProfit,OrderExpiration(),CLR_NONE);
         ReportAction(OrderTicket() + " profit moved " + dProfit);
         ReportOrder(res);
      }
   }
}

//+------------------------------------------------------------------+
//| TimeClose                                                           |
//+------------------------------------------------------------------+

void TimeClose( string _mtsName, int _ma, int TimeFrame, int HoursClose )
{
   for(int i=0;i<OrdersTotal();i++)
   {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false)        break;
      if( OrderMagicNumber()!=MagicMA(_ma) || OrderSymbol()!=Symbol()) continue;
      
      if( OrderSymbol()==Symbol() && Volume[0]<=1 && 
          iTime(Symbol(),TimeFrame,0) >= OrderOpenTime()+HoursClose*3600 )
      {
         switch( OrderType() )
         {
            case OP_BUY:
               AllowTrade();
               OrderClose(OrderTicket(),OrderLots(),Bid,3, DodgerBlue);
               ReportAction( "Closed " + OrderTicket() + " " + PX_NormPrice(Bid) );
               break;
            case OP_SELL:
               AllowTrade();
               OrderClose(OrderTicket(),OrderLots(),Ask,3, Yellow);
               ReportAction( "Closed " + OrderTicket() + " " + PX_NormPrice(Ask) );
               break;
         }
      }
   }
}

void StartTesting()
{
   if( IsTesting() )
      GlobalVariableDel( PXS_GetGlobalVarLO() );
}

int FindZigUP( int _pos )
{
   for( int i=1;i<120;i++ )
   {
      if( PX_TestFrahtalUP( i ) )
         return( i );
   }
      
   return( 0 );
}

int FindZigDown( int _pos )
{
   for( int i=1;i<120;i++ )
   {
      if( PX_TestFrahtalDown( i ) )
         return( i );
   }
      
   return( 0 );
}